import '../Stream.dart';
import '../Tag.dart';

class ByteArrayTag extends Tag {
  final List<int> value = [];

  ByteArrayTag();

  ByteArrayTag._(List<int> value) {
    this.value.addAll(value);
  }

  static ByteArrayTag valueOf(List<int> value) {
    return ByteArrayTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    int len = data.readInt();
    for (int i = 0; i < len; i++) {
      value.add(data.readByte());
    }
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeInt(value.length);
    for (int i in value) {
      data.writeByte(i);
    }
  }

  @override
  TagType getTagType() {
    return TagType.ByteArray;
  }

  @override
  dynamic getValue() {}

  @override
  void setValue(dynamic val) {}

  @override
  void prettyPrint(int indent, bool recurse) {
    String array = "";
    for (int b in value) {
      array += "$b, ";
    }
    array = array.substring(0, array.length - 2);

    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: [$array]");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append(
        "${isList ? "".padLeft(indent, '\t') : ""}[B; ${value.join("B, ")}B]");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    reader.expect("[");
    reader.expect("B");
    reader.expect(";");
    while (reader.peek() != "]") {
      value.add(int.parse(reader.readNumber()));
      reader.expect("b");

      if (reader.peek() == ",") reader.next();
    }
    reader.expect("]");
  }
}
