import '../Stream.dart';
import '../Tag.dart';

class ByteTag extends Tag {
  int value = 0;

  ByteTag._(int value) {
    this.value = value;
  }

  ByteTag();

  static ByteTag valueOf(int value) {
    return ByteTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readByte();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeByte(value);
  }

  @override
  TagType getTagType() {
    return TagType.Byte;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void setValue(val) {
    if (val is int) this.value = value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append("${value}b");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    String val = reader.readNumber();
    value = int.parse(val);

    reader.expect("b");
  }
}
