import '../Stream.dart';
import '../Tag.dart';

class CompoundTag extends Tag implements Map<String, Tag> {
  late final Map<String, Tag> value = {};

  CompoundTag();

  @override
  void readValue(ByteLayer data) {
    value.clear();

    //NBTAccountant.visitTag();
    //NBTAccountant.printRead(this);

    while (true) {
      Tag tag = Tag.readNamedTag(data);
      if (tag.getType() == 0) {
        //NBTAccountant.leaveTag(this);
        return;
      }

      tag.setParentTagType(TagType.Compound);
      put(tag.getKey(), tag);
/*
      if (tag.getTagType() != TagType.Compound &&
          tag.getTagType() != TagType.List) {
        NBTAccountant.visitTag();
        NBTAccountant.printRead(tag);
        NBTAccountant.leaveTag(tag);
      }*/
    }
  }

  @override
  void writeValue(ByteLayer data) {
    Iterator<Tag> it = value.values.iterator;

    while (it.moveNext()) {
      Tag tag = it.current;
      Tag.writeNamedTag(tag, data);
    }

    data.writeByte(TagType.End.byte);
  }

  void put(String name, Tag tag) {
    value[name] = tag;
    tag.setKey(name);
    tag.updateParent(this);
  }

  Tag? get(String name) {
    if (containsKey(name)) {
      return value[name] as Tag;
    } else {
      // Does not exist!
      return null;
    }
  }

  @override
  TagType getTagType() {
    return TagType.Compound;
  }

  @override
  getValue() {
    return null;
  }

  @override
  void setValue(val) {}

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: [${value.length} entries]");
    print("${"".padLeft(indent, '\t')}{");
    if (recurse) {
      for (Tag tag in value.values) {
        tag.prettyPrint(indent + 1, true);
      }
    }
  }

  void endPrettyPrint(int indent) {
    print("${"".padLeft(indent, '\t')}}");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    Iterator<Tag> it = value.values.iterator;
    builder.append("${isList ? "".padLeft(indent - 1, '\t') : ""}{\n");

    bool firstEntry = true;
    while (it.moveNext()) {
      Tag t = it.current;
      if (firstEntry) {
        firstEntry = false;
      } else {
        builder.append(",\n");
      }
      Tag.writeStringifiedNamedTag(t, builder, indent);
    }
    builder.append("\n${"".padLeft(indent - 1, '\t')}}");
  }

  @override
  Tag? operator [](Object? key) {
    return value[key];
  }

  @override
  void operator []=(String key, Tag value) {
    this.value[key] = value;
    value.updateParent(this);
  }

  @override
  void addAll(Map<String, Tag> other) {
    value.addAll(other);
    for (var entries in other.entries) {
      entries.value.updateParent(this);
    }
  }

  @override
  void addEntries(Iterable<MapEntry<String, Tag>> newEntries) {
    value.addEntries(newEntries);
    for (var entries in newEntries) {
      entries.value.updateParent(this);
    }
  }

  @override
  Map<RK, RV> cast<RK, RV>() {
    return value.cast();
  }

  @override
  void clear() {
    unsetParent();
    value.clear();
  }

  void unsetParent() {
    for (MapEntry<String, Tag> Entries in value.entries) {
      Entries.value.setParentTagType(TagType.End);
      Entries.value.updateParent(null);
    }
  }

  @override
  bool containsKey(Object? key) {
    return value.containsKey(key);
  }

  @override
  bool containsValue(Object? value) {
    return this.value.containsValue(value);
  }

  @override
  Iterable<MapEntry<String, Tag>> get entries => value.entries;

  @override
  void forEach(void Function(String key, Tag value) action) {
    value.forEach(action);
  }

  @override
  bool get isEmpty => value.isEmpty;

  @override
  bool get isNotEmpty => value.isNotEmpty;

  @override
  Iterable<String> get keys => value.keys;

  @override
  int get length => value.length;

  @override
  Map<K2, V2> map<K2, V2>(
      MapEntry<K2, V2> Function(String key, Tag value) convert) {
    return value.map(convert);
  }

  @override
  Tag putIfAbsent(String key, Tag Function() ifAbsent) {
    Tag X = this.value.putIfAbsent(key, ifAbsent);
    X.updateParent(this);
    return X;
  }

  @override
  Tag? remove(Object? key) {
    if (key is Tag && value.containsKey(key)) (key).updateParent(null);
    return value.remove(key);
  }

  /// DO NOT USE, Parent will not be updated on child tags when this function is used.
  @override
  void removeWhere(bool Function(String key, Tag value) test) {
    value.removeWhere(test);
  }

  /// DO NOT USE
  @override
  Tag update(String key, Tag Function(Tag value) update,
      {Tag Function()? ifAbsent}) {
    return value.update(key, update);
  }

  /// DO NOT USE
  @override
  void updateAll(Tag Function(String key, Tag value) update) {
    value.updateAll(update);
  }

  @override
  Iterable<Tag> get values => value.values;

  @override
  void readStringifiedValue(StringReader reader) {
    reader.expect("{");

    while (reader.peek() != "}") {
      Tag tag = Tag.readStringifiedNamedTag(reader);
      put(tag.getKey(), tag);

      if (reader.peek() == ",") reader.next();
    }

    reader.expect("}");
  }
}
