import 'package:intl/intl.dart';

import '../Stream.dart';
import '../Tag.dart';

class DoubleTag extends Tag {
  double value = 0.0;

  DoubleTag();

  DoubleTag._(double value) {
    this.value = value;
  }

  static DoubleTag valueOf(double value) {
    return DoubleTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readDouble();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeDouble(value);
  }

  @override
  TagType getTagType() {
    return TagType.Double;
  }

  @override
  void setValue(val) {
    if (val is double) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: ${value.toString()}");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append(
        "${isList ? "".padLeft(indent, '\t') : ""}${value.toString()}d");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    double val = double.parse(reader.readNumber());
    value = val;

    reader.expect("d");
  }
}
