import '../Stream.dart';
import '../Tag.dart';

class IntTag extends Tag {
  int value = 0;
  IntTag();

  IntTag._(int value) {
    this.value = value;
  }

  static IntTag valueOf(int value) {
    return IntTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readInt();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeInt(value);
  }

  @override
  TagType getTagType() {
    return TagType.Int;
  }

  @override
  void setValue(val) {
    if (val is int) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append("${isList ? "".padLeft(indent, '\t') : ""}${value}i");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    String val = reader.readNumber();
    value = int.parse(val);

    // Since a type indicator is optional for a int, check for a comma
    if (reader.peek() == ",")
      return;
    else
      reader.expect("i");
  }
}
