import '../Stream.dart';
import '../Tag.dart';
import 'EndTag.dart';

class ListTag extends Tag {
  List<Tag> value = [];

  ListTag();

  TagType get getListTagType =>
      value.length == 0 ? TagType.End : value[0].getTagType();

  @override
  void readValue(ByteLayer data) {
    TagType type = TagType.get(data.readByte());
    int size = data.readInt();

    for (int i = 0; i < size; i++) {
      Tag tag = Tag.makeTagOfType(type);
      tag.readValue(data);
      add(tag);
    }
  }

  @override
  void writeValue(ByteLayer data) {
    TagType type = TagType.End;
    if (size() > 0) {
      type = value[0].getTagType();
    }

    data.writeByte(type.byte);
    data.writeInt(size());
    for (int i = 0; i < size(); i++) {
      value[i].writeValue(data);
    }
  }

  void add(Tag tag) {
    TagType type = TagType.End;
    if (size() > 0) {
      type = value[0].getTagType();
    }
    if (type == TagType.End || type == tag.getTagType()) {
      value.add(tag);
      tag.updateParent(this);
    }
  }

  Tag get(int index) {
    if (size() > index) {
      return value[index];
    } else {
      return EndTag();
    }
  }

  void remove(Tag tag) {
    value.remove(tag);
    tag.updateParent(null);
  }

  void removeAt(int index) {
    value[index].updateParent(null);
    value.removeAt(index);
  }

  int indexOf(Tag tag) {
    return value.indexOf(tag);
  }

  @override
  TagType getTagType() {
    return TagType.List;
  }

  @override
  void setValue(val) {}

  @override
  getValue() {}

  int size() {
    return value.length;
  }

  void clear() {
    // Clear the list
    for (var entry in value) {
      entry.updateParent(null);
      entry.setParentTagType(TagType.End);
    }

    value.clear();
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: [${value.length} entries]");
    print("${"".padLeft(indent, '\t')}[");
    for (Tag tag in value) {
      tag.prettyPrint(indent + 1, true);
    }
  }

  void endPrettyPrint(int indent) {
    print("${"".padLeft(indent, '\t')}]");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append("${isList ? "".padLeft(indent - 1, '\t') : ""}[\n");
    Iterator<Tag> it = value.iterator;
    bool firstTag = true;
    while (it.moveNext()) {
      Tag tag = it.current;
      if (firstTag)
        firstTag = !firstTag;
      else {
        builder.append(",\n");
      }

      tag.writeStringifiedValue(builder, indent + 1, true);
    }
    builder.append("\n${"".padLeft(indent - 1, '\t')}]");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    reader.expect("[");
    while (reader.peek() != "]") {
      TagType type = TagType.getStringifiedTagType(reader);
      Tag newTag = Tag.makeTagOfType(type);
      newTag.readStringifiedValue(reader);
      add(newTag);

      if (reader.peek() == ",") reader.next();
    }
    reader.expect("]");
  }
}
