import '../Stream.dart';
import '../Tag.dart';

class LongArrayTag extends Tag {
  final List<int> value = [];
  LongArrayTag();

  LongArrayTag._(List<int> lst) {
    value.addAll(lst);
  }

  static LongArrayTag valueOf(List<int> value) {
    return LongArrayTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    int count = data.readInt();
    for (int i = 0; i < count; i++) {
      value.add(data.readLong());
    }
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeInt(size());
    for (int i in value) {
      data.writeLong(i);
    }
  }

  int size() {
    return value.length;
  }

  @override
  TagType getTagType() {
    return TagType.LongArray;
  }

  @override
  void setValue(val) {}

  @override
  getValue() {}

  @override
  void prettyPrint(int indent, bool recurse) {
    String array = "";
    for (int b in value) {
      array += "$b, ";
    }
    array = array.substring(0, array.length - 2);

    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: [$array]");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append(
        "${isList ? "".padLeft(indent, '\t') : ""}[L; ${value.join('L, ')}L]");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    reader.expect("[");
    reader.expect("L");
    reader.expect(";");
    while (reader.peek() != "]") {
      value.add(int.parse(reader.readNumber()));
      reader.expect("l");

      if (reader.peek() == ",") reader.next();
    }
    reader.expect("]");
  }
}
