import '../Stream.dart';
import '../Tag.dart';

class LongTag extends Tag {
  int value = 0;
  LongTag();

  LongTag._(int value) {
    this.value = value;
  }

  static LongTag valueOf(int value) {
    return LongTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readLong();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeLong(value);
  }

  @override
  TagType getTagType() {
    return TagType.Long;
  }

  @override
  void setValue(val) {
    if (val is int) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append("${isList ? "".padLeft(indent, '\t') : ""}${value}L");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    int val = int.parse(reader.readNumber());
    value = val;

    reader.expect("l");
  }
}
