import '../Stream.dart';
import '../Tag.dart';

class ShortTag extends Tag {
  int value = 0;
  ShortTag();

  ShortTag._(int value) {
    this.value = value;
  }

  static ShortTag valueOf(int value) {
    return ShortTag._(value);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readShort();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeShort(value);
  }

  @override
  TagType getTagType() {
    return TagType.Short;
  }

  @override
  void setValue(val) {
    if (val is int) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    builder.append("${isList ? "".padLeft(indent, '\t') : ""}${value}S");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    int val = int.parse(reader.readNumber());
    value = val;

    reader.expect("s");
  }
}
