import '../Stream.dart';
import '../Tag.dart';

class StringTag extends Tag {
  String value = "";
  StringTag();

  StringTag._(String str) {
    value = str;
  }

  static StringTag valueOf(String str) {
    return StringTag._(str);
  }

  @override
  void readValue(ByteLayer data) {
    value = data.readString();
  }

  @override
  void writeValue(ByteLayer data) {
    data.writeString(value);
  }

  @override
  TagType getTagType() {
    return TagType.String;
  }

  @override
  void setValue(val) {
    if (val is String) value = val;
  }

  @override
  getValue() {
    return value;
  }

  @override
  void prettyPrint(int indent, bool recurse) {
    print(
        "${"".padLeft(indent, '\t')}${Tag.getCanonicalName(getTagType())}: $value");
  }

  @override
  void writeStringifiedValue(StringBuilder builder, int indent, bool isList) {
    final useSingleQuotes = shouldUseSingleQuotes(value);
    final quote = useSingleQuotes ? '\'' : '"';
    final escapeQuote = useSingleQuotes ? '\\\'' : '\\"';

    String escapedValue = value;
    if (shouldEscapeSingleQuotes(value) && useSingleQuotes) {
      escapedValue = value.replaceAll('\'', escapeQuote);
    }

    builder.append(
        "${isList ? "".padLeft(indent, '\t') : ""}${quote}${escapedValue}${quote}");
  }

  @override
  void readStringifiedValue(StringReader reader) {
    String str = reader.readString();
    value = str;
  }
}
