import 'dart:math';

class SLFloat {
  dynamic objValue;

  double get value {
    objValue ??= 0.0;
    return objValue;
  }

  set value(double val) {
    objValue = val;
  }

  SLFloat(double a) {
    objValue = a;
  }

  SLFloat.fromString(String s) {
    final regex = RegExp(
        r'\A[ ]*(?<sign>[+-]?)'
        r'(?:'
        r'(?:'
        r'0x'
        r'(?:'
        r'(?<int>[0-9a-f]*)'
        r'(?:'
        r'(?:\.(?<frac>[0-9a-f]*))?'
        r')?'
        r'(?:p(?<exp>[-+]?[\d]*))?'
        r')?'
        r')'
        r'|'
        r'(?<dec>[\d]*[.]?[\d]*'
        r'(?:e[+-]?[\d]*)?'
        r')'
        r')',
        caseSensitive: false);

    final match = regex.firstMatch(s);
    double mantissa = 0.0;

    if (match?.namedGroup('dec')?.isNotEmpty ?? false) {
      mantissa = double.parse(match!.namedGroup('dec')!);
    }

    if (match?.namedGroup('int')?.isNotEmpty ?? false) {
      mantissa = int.parse(match!.namedGroup('int')!, radix: 16).toDouble();

      if (match.namedGroup('frac')?.isNotEmpty ?? false) {
        mantissa += int.parse(match.namedGroup('frac')!, radix: 16) /
            pow(16.0, match.namedGroup('frac')!.length);
      }

      if (match.namedGroup('exp')?.isNotEmpty ?? false) {
        mantissa *= pow(2.0, int.parse(match.namedGroup('exp')!));
      }
    }

    objValue = (match?.namedGroup('sign') == '-') ? -mantissa : mantissa;
  }

  @override
  int get hashCode => value.hashCode;

  @override
  String toString() => value.toStringAsFixed(6);

  static String explicitString(SLFloat x) => x.toString();

  static double implicitDouble(SLFloat x) => x.value;

  static SLFloat fromStringExplicit(String s) => SLFloat.fromString(s);

  static SLFloat fromInt(int x) => SLFloat(x.toDouble());

  static SLFloat fromLong(int x) => SLFloat(x.toDouble());

  static SLFloat fromDouble(double x) => SLFloat(x);

  SLFloat multiply(SLFloat other) => SLFloat(value * other.value);

  SLFloat divide(SLFloat other) => SLFloat(value / other.value);

  SLFloat add(SLFloat other) => SLFloat(value + other.value);

  SLFloat subtract(SLFloat other) => SLFloat(value - other.value);

  SLFloat increment() => SLFloat(value + 1.0);

  SLFloat decrement() => SLFloat(value - 1.0);

  bool equals(SLFloat other) =>
      (value - other.value).abs() < double.minPositive;

  bool notEquals(SLFloat other) => !equals(other);

  int compareTo(SLFloat other) => value.compareTo(other.value);

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    return other is SLFloat && equals(other);
  }
}
