class Integer {
  static const int _minValue = -2147483648;
  static const int _maxValue = 2147483647;
  int _value = 0;

  static final Integer FALSE = Integer(0);
  static final Integer TRUE = Integer(1);

  int get value => _value;
  set value(int v) => _value = _clampTo32Bit(v);

  // Constructor
  Integer(int value) : _value = _clampTo32Bit(value);

  Integer.fromString(String s) {
    final f = double.parse(s);
    _value = _clampTo32Bit(f.toInt());
  }

  // Implicit conversions (in Dart these are done with factory constructors)
  factory Integer.fromBool(bool x) => x ? TRUE : FALSE;
  factory Integer.fromInt(int x) => Integer(x);
  factory Integer.fromDouble(double x) => Integer(_clampTo32Bit(x.toInt()));
  factory Integer.fromStringExplicit(String x) => Integer.fromString(x);

  // Explicit conversion to bool
  bool toBool() => value != 0;

  // Logical negation (NOT) operator
  /// Functions the same as Second Life, where if positive, returns false, if 0/false, then return true.
  Integer not() {
    if (value > 0)
      return FALSE;
    else
      return TRUE;
  }

  // Bitwise AND operator
  Integer operator &(Integer y) => Integer(value & y.value);

  // Bitwise OR operator
  Integer operator |(Integer y) => Integer(value | y.value);

  // Bitwise NOT operator
  Integer operator ~() => Integer(~value);

  // Bitwise XOR operator
  Integer operator ^(Integer b) => Integer(value ^ b.value);

  // Bitwise Shift right
  Integer operator >>(int b) => Integer(value >> b);

  // Bitwise Shift left
  Integer operator <<(int b) => Integer(_clampTo32Bit(value << b));

  // Modulus operator
  Integer operator %(Integer b) => Integer(value % b.value);

  // Definitely true operator
  bool get isTrue => value != 0;

  // Definitely false operator
  bool get isFalse => value == 0;

  // Explicit conversion to String
  String toStringExplicit() => value.toString();

  // Implicit conversion to int
  int toInt() => value;

  // Addition operator
  Integer operator +(Integer b) => Integer(value + b.value);

  // Subtraction operator
  Integer operator -(Integer b) => Integer(value - b.value);

  /*
  Integer operator -(bool b) => this - Integer.fromBool(b);
  Integer operator +(bool b) => this + Integer.fromBool(b);
  */

  // Multiplication operator
  Integer operator *(Integer b) => Integer(value * b.value);

  Integer multiplyByDouble(double b) {
    final result = value * b;
    return Integer(result.toInt());
  }

  // Equality operators
  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is Integer &&
          runtimeType == other.runtimeType &&
          value == other.value;

  @override
  int get hashCode => value.hashCode;

  //bool operator !=(Object other) => !this.==(other);

  // Comparison method
  static int compare(Integer a, Integer b) {
    if (a.value < b.value) return -1;
    if (a.value > b.value) return 1;
    return 0;
  }

  // Override toString
  @override
  String toString() => value.toString();

  // Helper method to clamp the value to a 32-bit signed integer range
  static int _clampTo32Bit(int v) {
    if (v < _minValue) return _minValue;
    if (v > _maxValue) return _maxValue;
    return v;
  }
}
