class Rotation {
  double x = 0;
  double y = 0;
  double z = 0;
  double s = 0;

  static final Rotation zeroRotation = Rotation(0, 0, 0, 1);

  Rotation(this.x, this.y, this.z, this.s);

  Rotation.fromString(String a) {
    final regex = RegExp(r'<(?<x>[^,]*),(?<y>[^,]*),(?<z>[^,]*),(?<s>[^,]*)>');
    final match = regex.firstMatch(a);

    if (match != null) {
      x = double.parse(match.namedGroup('x') ?? '0');
      y = double.parse(match.namedGroup('y') ?? '0');
      z = double.parse(match.namedGroup('z') ?? '0');
      s = double.parse(match.namedGroup('s') ?? '0');
    }
  }

  @override
  String toString() {
    return "<${x.toStringAsFixed(5)}, ${y.toStringAsFixed(5)}, ${z.toStringAsFixed(5)}, ${s.toStringAsFixed(5)}>";
  }

  Rotation operator -() {
    return Rotation(-x, -y, -z, -s);
  }

  Rotation operator +(Rotation other) {
    return Rotation(x + other.x, y + other.y, z + other.z, s + other.s);
  }

  Rotation operator *(Rotation other) {
    return Rotation(
      other.s * x - other.z * y + other.y * z + other.x * s,
      other.s * y + other.z * x + other.y * s - other.x * z,
      other.s * z + other.z * s - other.y * x + other.x * y,
      other.s * s - other.z * z - other.y * y - other.x * x,
    );
  }

  Rotation operator /(Rotation other) {
    return Rotation(
      other.s * x + other.z * y - other.y * z - other.x * s,
      other.s * y - other.z * x - other.y * s + other.x * z,
      other.s * z - other.z * s + other.y * x - other.x * y,
      other.s * s + other.z * z + other.y * y + other.x * x,
    );
  }

  /// Epsilon
  static const equalityTolerance = 1e-14;

  bool operator ==(Object other) {
    if (other is Rotation) {
      return (x - other.x).abs() <= equalityTolerance &&
          (y - other.y).abs() <= equalityTolerance &&
          (z - other.z).abs() <= equalityTolerance &&
          (s - other.s).abs() <= equalityTolerance;
    }
    return false;
  }

  @override
  int get hashCode => x.hashCode ^ y.hashCode ^ z.hashCode ^ s.hashCode;

  bool isTrue() {
    return !(x == 0 && y == 0 && z == 0 && s == 1);
  }

  bool isFalse() {
    return x == 0 && y == 0 && z == 0 && s == 1;
  }
}
