import '../../utils/Vector3.dart';

class Vector {
  Vector3d _vector = Vector3d.ZERO;

  static final Vector ZERO_VECTOR = Vector(0, 0, 0);

  double get x => _vector.X;
  set x(double value) => _vector.X = value;

  double get y => _vector.Y;
  set y(double value) => _vector.Y = value;

  double get z => _vector.Z;
  set z(double value) => _vector.Z = value;

  Vector(double x, double y, double z) : _vector = Vector3d(X: x, Y: y, Z: z);

  Vector.fromString(String s) {
    final regex = RegExp(r'<(?<x>[^,]*),(?<y>[^,]*),(?<z>[^,]*)>');
    final match = regex.firstMatch(s);
    if (match != null) {
      _vector = Vector3d(
        X: double.parse(match.namedGroup('x') ?? '0'),
        Y: double.parse(match.namedGroup('y') ?? '0'),
        Z: double.parse(match.namedGroup('z') ?? '0'),
      );
    } else {
      _vector = Vector3d(X: 0, Y: 0, Z: 0);
    }
  }

  double _sumSqrs() {
    return x * x + y * y + z * z;
  }

  Vector add(Vector other) {
    return Vector(x + other.x, y + other.y, z + other.z);
  }

  Vector subtract(Vector other) {
    return Vector(x - other.x, y - other.y, z - other.z);
  }

  Vector negate() {
    return Vector(-x, -y, -z);
  }

  double dot(Vector other) {
    return x * other.x + y * other.y + z * other.z;
  }

  Vector divide(double value) {
    return Vector(x / value, y / value, z / value);
  }

  Vector multiply(double value) {
    return Vector(x * value, y * value, z * value);
  }

  Vector cross(Vector other) {
    return Vector(
      y * other.z - z * other.y,
      z * other.x - x * other.z,
      x * other.y - y * other.x,
    );
  }

  bool lessThan(Vector other) {
    return _sumSqrs() < other._sumSqrs();
  }

  bool lessThanOrEqual(Vector other) {
    return _sumSqrs() <= other._sumSqrs();
  }

  bool greaterThan(Vector other) {
    return _sumSqrs() > other._sumSqrs();
  }

  bool greaterThanOrEqual(Vector other) {
    return _sumSqrs() >= other._sumSqrs();
  }

  static const double equalityTolerance = 1e-14;

  @override
  bool operator ==(Object other) {
    if (other is Vector) {
      return (x - other.x).abs() <= equalityTolerance &&
          (y - other.y).abs() <= equalityTolerance &&
          (z - other.z).abs() <= equalityTolerance;
    }
    return false;
  }

  @override
  int get hashCode => (x + y + z).toInt() % 2147483647; // Int32.MaxValue in C#

  @override
  String toString() {
    return '<${x.toStringAsFixed(5)}, ${y.toStringAsFixed(5)}, ${z.toStringAsFixed(5)}>';
  }
}
